function p=readasl2008(fnameroot,graphics,dorblinks,nocr)
% reads an ASL 2006 file
% usage: p=readasl2006(fnameroot,graphics,dorblinks,nocr)
%   last parameter kills the corneal reflection signal.
%   good for use with fMRI where it's not stable.
%   assumes we're going to use head motion for correction.
% note - for this to work we need ASLFileLib2_x64.dll to be registered
% do this by running a command prompt as administrator and typing:
%  cd c:\Program Files (x86)\Applied Science Laboratories\DLL
%  regsvr32 ASLFileLib2_x64.dll


% Create Automation server
if nargin<2, graphics=1; end
if nargin<3, dorblinks=1; end
if nargin<4, nocr=1; end


fname=sprintf('%s.eyd',fnameroot);
p.fname=fname;
fprintf('Reading file: %s\n',fname);

[h.freq,h.segments,h.scale_factors,h.items,h.itemNames]=ASLReadFileHeader(fname);
p.header=h;
p.RescaleFactor=p.header.freq; % was 1./p.header.freq
svr = actxserver('ASLFileLib2.ASLFile');

% Open file
[segmentCount, itemNames] = invoke(svr, 'OpenFileForReading', fname);
p.header.DataItemNames=itemNames;
itemNames(2)=cellstr('video_field');


% Read first record
ct=1;
try,
  while (1)
    record = svr.ReadDataRecord;
    p.raw(ct)=cell2struct(record,itemNames);
    % initialize the structure to a LARGE size (60 minutes of data)
    % so that we don't have to reallocate memory much
    if ct==1
      p.raw=repmat(p.raw(1),216000,1);
    end
    ct=ct+1;
    if mod(ct,500)==0, 
      fprintf('.',ct); 
    end
  end
catch,
  fprintf('\nread %d records\n',ct-1);
end
p.raw=p.raw(1:ct);

% Close file
svr.CloseFile

% Release the memory
svr.delete
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% correct raw data for times not acquired so that samples
% are acquired at regular intervals
% need to correct: 
%   p.raw.pupil_diam
%   p.raw.pupil_recogn
%   p.raw.pupil_pos_horz
%   p.raw.cr_pos_horz
%   p.raw.pupil_pos_vert
%   p.raw.cr_pos_vert
%   p.raw.XDAT
% horz & vert gaze coord

% based on p.raw.total_secs
%   a=[p.raw.total_secs];
%   b=a(2:end)-a(1:end-1);
% 
% assume that the data is collected at 60Hz
samps=[p.raw.total_secs].*60;
samps=round(samps-samps(1)+1);
% must fix this to interpolate only WITHIN blocks
% so if there's a wait of >100 samples remove it
seg=[p.raw.segment];
blockbreaks=1+find(seg(2:end)-seg(1:end-1));
%sampdiffs=samps(2:end)-samps(1:end-1);
%blockbreaks=(1+find(sampdiffs>100));
if length(blockbreaks>0)
  p.blockbreaks=blockbreaks;
  p.blocksubinds=samps(p.blockbreaks)-samps(p.blockbreaks-1);
  for ct=1:(length(blockbreaks)-1);
    samps(blockbreaks(ct):blockbreaks(ct+1)-1)=samps(blockbreaks(ct):blockbreaks(ct+1)-1)-samps(blockbreaks(ct))+samps(blockbreaks(ct)-1)+1;
  end
end

p.raw2.total_secs(samps)=[p.raw.total_secs];
p.raw2.pupil_diam(samps)=[p.raw.pupil_diam];
p.raw2.pupil_recogn(samps)=[p.raw.pupil_recogn];
%p.raw2.pupil_pos_horz(samps)=[p.raw.pupil_pos_horz];
%p.raw2.cr_pos_horz(samps)=[p.raw.cr_pos_horz];
%p.raw2.pupil_pos_vert(samps)=[p.raw.pupil_pos_vert];
%p.raw2.cr_pos_vert(samps)=[p.raw.cr_pos_vert];
% rescaling at this step can be helpful. not sure though....
% note: VOIDS blink detection...
%if isfield(p.raw(1),'pupil_pos_horz')
%  p.raw2.pupil_pos_horz(samps)=rescaleoutlierstimeseries([p.raw.pupil_pos_horz]);
%  p.raw2.pupil_pos_vert(samps)=rescaleoutlierstimeseries([p.raw.pupil_pos_vert]);
%else
p.raw2.pupil_pos_horz(samps)=rescaleoutlierstimeseries([p.raw.horz_gaze_coord]);
p.raw2.pupil_pos_vert(samps)=rescaleoutlierstimeseries([p.raw.vert_gaze_coord]);
%end  
if ~nocr & isfield(p.raw,'cr_pos_horz')
  p.raw2.cr_pos_horz(samps)=rescaleoutlierstimeseries([p.raw.cr_pos_horz]);
  p.raw2.cr_pos_vert(samps)=rescaleoutlierstimeseries([p.raw.cr_pos_vert]);
end
p.raw2.XDAT(samps)=[p.raw.XDAT];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
p.RescaleData=[p.raw2.pupil_diam];

% identify blinks from the ASL file
p.BlinkTimes=1-[p.raw2.pupil_recogn];
% stuff too low to be real is a blink
%p.BlinkTimes=union(p.BlinkTimes,find(p.RescaleData<(prctile(p.RescaleData(find(p.RescaleData>0)),25)-1.*iqr(p.RescaleData(find(p.RescaleData>0))))));

p.BlinkTimes=p.BlinkTimes | (p.RescaleData<(prctile(p.RescaleData,25)-3.*iqr(p.RescaleData)));

b=conv(double(p.BlinkTimes),double([1 1 1 1 1 1 1 1 1])); 
p.BlinkTimes=b(2:end-1)>0;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% more blink identification

% fill in blinks which are too close
btrans=find(p.BlinkTimes(2:end)-p.BlinkTimes(1:end-1));
bstarts=btrans(1:2:end);
bends=btrans(2:2:end);
bstarts=bstarts(1:length(bends));
bdists=bstarts(2:end)-bends(1:end-1);
tooclose=find(bdists<11);
toocloseends=btrans(tooclose.*2);
tooclosenextstarts=btrans(1+tooclose.*2);
for ct=1:length(toocloseends)
  p.BlinkTimes(toocloseends(ct):tooclosenextstarts(ct))=1;
end

% fill in areas with big transitions next to blinks
%btrans=find(p.BlinkTimes(2:end)-p.BlinkTimes(1:end-1));
%bstarts=btrans(1:2:end);
%bends=btrans(2:2:end);
%notdoneyet=find(abs(p.RescaleData(bends)<(p.RescaleData(bends+5)+10)));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nocr
  p.XmCRX=([p.raw2.pupil_pos_horz])';
  p.YmCRY=([p.raw2.pupil_pos_vert])';  
else
  p.XmCRX=([p.raw2.pupil_pos_horz]-[p.raw2.cr_pos_horz])';
  p.YmCRY=([p.raw2.pupil_pos_vert]-[p.raw2.cr_pos_vert])';
end
p.X=[p.raw2.pupil_pos_horz]';
p.Y=[p.raw2.pupil_pos_vert]';

allxdat=[p.raw2.XDAT]-median([p.raw2.XDAT]);

[bog,tmpEventTicks]=find(allxdat);
if length(tmpEventTicks)
  p.EventTicks(1)=tmpEventTicks(1);
end
if length(tmpEventTicks>1)
  enum=2;
  for ct=2:length(tmpEventTicks)
    if tmpEventTicks(ct)~=tmpEventTicks(ct-1)+1
      p.EventTicks(enum)=tmpEventTicks(ct);
      enum=enum+1;
    end
  end
end
if isfield(p,'EventTicks')
  p.EventCodes=double(allxdat(p.EventTicks)');
  p.EventTimes=p.EventTicks'./p.header.freq;
else
  p.EventTicks=[];
  p.EventCodes=[];
  p.EventTimes=[];
end

% remove blinks
if dorblinks
  % this code would do it Eric's way
  %  avgdata=dbfilter(data.RescaleData,5)'; % smooth 
  %  avgdata=avgdata(1:numpts);
  %  data.NoBlinks=rblinks(avgdata,.3,.2,1.0,10,7)'; % 
  %  data.BlinkTimes=avgdata~=data.NoBlinks;
  
  % this does it my way, with a second pass
  %p=puppreprocess(p,.3,.2,1.0,10,7);

  

  % clean data a little by linear interpolations through blinks
  p.BlinkTimesXY=1-[p.raw2.pupil_recogn]';
  p.BlinkTimesXY=p.BlinkTimesXY | (p.XmCRX<(prctile(p.XmCRX,25)-3.*iqr(p.XmCRX)));
  p.BlinkTimesXY=p.BlinkTimesXY | (p.XmCRX>(prctile(p.XmCRX,75)+3.*iqr(p.XmCRX)));
  p.BlinkTimesXY=p.BlinkTimesXY | (p.YmCRY<(prctile(p.YmCRY,25)-3.*iqr(p.YmCRY)));
  p.BlinkTimesXY=p.BlinkTimesXY | (p.YmCRY>(prctile(p.YmCRY,75)+3.*iqr(p.YmCRY)));
  
  b=conv(double(p.BlinkTimesXY),double([1 1 1 1 1 1 1 1 1])); 
  p.BlinkTimesXY=b(4:end-5)>0;
  % interpolate eye trace through blinks
  p.XmCRXn=p.XmCRX; p.YmCRYn=p.YmCRY;
  p.XmCRXn(p.BlinkTimesXY)=-999; p.XmCRXn=interpmissing(p.XmCRXn,-999);
  p.YmCRYn(p.BlinkTimesXY)=-999; p.YmCRYn=interpmissing(p.YmCRYn,-999);
  p.NoBlinks=p.RescaleData';
  p.NoBlinks(p.BlinkTimesXY)=-999; p.NoBlinks=interpmissing(p.NoBlinks,-999);
  % apply eye calibration to get to screen coordinates
  p.XmCRX=p.XmCRXn; % note - this makes applyical use the blink-corrected data
  p.YmCRY=p.YmCRYn;    
  
  
  unscalednoblinks=p.NoBlinks;
  
  % get 95% trimmed mean
  sw=sort(p.NoBlinks);
  minsw=sw(round(.05.*length(sw))); 
  maxsw=sw(round(.95.*length(sw)));
  p.NoBlinksUnsmoothed=(p.NoBlinks-minsw)./(maxsw-minsw);
  p.RescaleData=(p.RescaleData-minsw)./(maxsw-minsw);
  p.NoBlinks=filter3p86hz(p.NoBlinksUnsmoothed')'; % filter at 3.86Hz = 5pt moving average

else
  p.NoBlinks=p.RescaleData';
end

% now convert to screen coordinates
p.XScreenCoords=(p.XmCRX./160).*640;
p.YScreenCoords=480-(p.YmCRY./240).*480;
